@@ -17,10 +17,10 @@
      may be used to endorse or promote products derived from this software
      without specific prior written permission.
 
-  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
@@ -49,17 +49,17 @@ struct attribute_tag {
 };
 
 
-void init_escseq(int use_civis, int use_ins_del, int status_type, const struct attribute_tag *attr_uim);
+void init_escseq(const struct attribute_tag *attr_uim);
 void quit_escseq(void);
+void fixtty(void);
 struct point_tag get_cursor_position(void);
+void put_move_cur(int from, int to);
+void put_cursor_left(int n);
+void put_cursor_right(int n);
 void put_save_cursor(void);
 void put_restore_cursor(void);
 void put_cursor_invisible(void);
 void put_cursor_normal(void);
-void put_enter_underline_mode(void);
-void put_exit_underline_mode(void);
-void put_enter_standout_mode(void);
-void put_exit_standout_mode(void);
 void put_exit_attribute_mode(void);
 void put_cursor_address(int row, int col);
 void put_cursor_address_p(struct point_tag *p);
@@ -70,9 +70,11 @@ void put_goto_lastline(int col);
 void put_erase(int n);
 void put_clear_to_end_of_line(int width);
 void put_change_scroll_region(int start, int end);
-void put_uim_str(const char *str);
-void put_uim_str_len(const char *str, int len);
-void put_pty_str(const char *str);
+void put_uim_str(const char *str, int attr);
+void put_uim_str_len(const char *str, int attr, int len);
+void put_uim_str_no_color(const char *str, int attr);
+void put_uim_str_no_color_len(const char *str, int attr, int len);
+void put_pty_str(const char *str, int len);
 char *cut_padding(const char *escseq);
 void escseq_winch(void);
 #endif
