@@ -35,7 +35,7 @@
 #define KEY_H
 int tty2key(char key);
 int tty2key_state(char key);
-int *escape_sequence2key(const char *str);
+int *escape_sequence2key(const char *str, int str_len);
 void print_key(int key, int key_state);
 
 #endif
