@@ -17,10 +17,10 @@
      may be used to endorse or promote products derived from this software
      without specific prior written permission.
 
-  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
@@ -31,34 +31,37 @@
 
 */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 #include <stdio.h>
 #ifndef DEBUG
 #define NDEBUG
 #endif
-#if HAVE_ASSERT_H
+#ifdef HAVE_ASSERT_H
 #include <assert.h>
 #endif
-#if HAVE_LOCALE_H
+#ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
-#if HAVE_WCHAR_H
+#ifdef HAVE_WCHAR_H
 #include <wchar.h>
 #endif
-#if HAVE_STRING_H
+#ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#if HAVE_STRINGS_H
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif
-#if HAVE_CTYPE_H
+#ifdef HAVE_CTYPE_H
 #include <ctype.h>
 #endif
-#if HAVE_STDLIB_H
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_LANGINFO_CODESET
+#include <langinfo.h>
+#endif
 #include "uim-fep.h"
 #include "str.h"
 
@@ -72,7 +75,7 @@ void init_str(void)
     printf("locale not supported\n");
     exit(1);
   }
-  s_utf8 = (strcasecmp(get_enc(), "utf-8") == 0 || strcasecmp(get_enc(), "utf8") == 0);
+  s_utf8 = (strcasecmp(get_enc(), "UTF-8") == 0 || strcasecmp(get_enc(), "UTF8") == 0);
 }
 
 /*
@@ -81,19 +84,23 @@ void init_str(void)
  */
 const char *get_enc(void)
 {
-  const char *locale;
-  if ((locale = getenv("LC_ALL")) ||
-      (locale = getenv("LC_CTYPE")) ||
-      (locale = getenv("LANG")))
-  {
-    if (strcasecmp(locale, "ja") == 0) {
-      return "euc-jp";
-    } else {
-      char *ptr = strstr(locale, ".");
-      return ptr != NULL ? ptr + 1 : "utf-8";
-    }
+#ifdef __CYGWIN32__
+  return "EUC-JP";
+#else
+#ifdef HAVE_LANGINFO_CODESET
+  return nl_langinfo(CODESET);
+#else
+  const char *locale = setlocale(LC_CTYPE, NULL);
+  assert(locale != NULL);
+
+  if (strcasecmp(locale, "ja") == 0) {
+    return "EUC-JP";
+  } else {
+    char *ptr = strstr(locale, ".");
+    return ptr != NULL ? ptr + 1 : "UTF-8";
   }
-  return "utf-8";
+#endif
+#endif
 }
 
 /*
@@ -150,7 +157,7 @@ int compare_str_rev(const char *str1, const char *str2)
  * strwidth("a") = 3
  * strwidth("")    = 0
  */
-#ifdef HAVE_WCSWIDTH
+#if defined(HAVE_WCSWIDTH) && !defined(__CYGWIN32__)
 int strwidth(const char *str)
 {
   int width;
@@ -177,7 +184,7 @@ int strwidth(const char *str)
 {
   int width = 0;
   for (; *str != '\0'; str++) {
-    if (isascii(*str)) {
+    if (isascii((unsigned char)*str)) {
       width++;
     } else {
       if (s_utf8) {
@@ -212,7 +219,7 @@ int strwidth(const char *str)
  * const char *strȤʤäƤ뤬Ū˽񴹤Τstrʸ
  * ˤƤϤʤ
  */
-#ifdef HAVE_WCSWIDTH
+#if defined(HAVE_WCSWIDTH) && !defined(__CYGWIN32__)
 int byte2width(const char *str, int n)
 {
   int width;
@@ -268,7 +275,7 @@ int byte2width(const char *str, int n)
   }
 
   for (; *str != '\0'; str++) {
-    if (isascii(*str)) {
+    if (isascii((unsigned char)*str)) {
       char_width = 1;
       char_byte = 1;
     } else {
@@ -313,7 +320,7 @@ int byte2width(const char *str, int n)
  * const char *strȤʤäƤ뤬Ū˽񴹤Τstrʸ
  * ˤƤϤʤ
  */
-#ifdef HAVE_WCSWIDTH
+#if defined(HAVE_WCSWIDTH) && !defined(__CYGWIN32__)
 int byte2width2(const char *str, int n)
 {
   int width;
@@ -372,7 +379,7 @@ int byte2width2(const char *str, int n)
   }
 
   for (; *str != '\0'; str++) {
-    if (isascii(*str)) {
+    if (isascii((unsigned char)*str)) {
       char_width = 1;
       char_byte = 1;
     } else {
@@ -411,7 +418,7 @@ int byte2width2(const char *str, int n)
  * width2byte("", 3) = [3, 2] (utf8)
  * width2byte("", 4) = [6, 4] (utf8)
  */
-#ifdef HAVE_WCSWIDTH
+#if defined(HAVE_WCSWIDTH) && !defined(__CYGWIN32__)
 int *width2byte(const char *str, int n)
 {
   int width = 0;
@@ -464,7 +471,7 @@ int *width2byte(const char *str, int n)
   static int rval[2];
 
   for (; *str != '\0'; str++) {
-    if (isascii(*str)) {
+    if (isascii((unsigned char)*str)) {
       char_width = 1;
       char_byte = 1;
     } else {
@@ -509,7 +516,7 @@ int *width2byte(const char *str, int n)
  * width2byte2("", 1) = [3, 2] (utf8)
  * width2byte2("", 4) = [6, 4] (utf8)
  */
-#ifdef HAVE_WCSWIDTH
+#if defined(HAVE_WCSWIDTH) && !defined(__CYGWIN32__)
 int *width2byte2(const char *str, int n)
 {
   int width = 0;
@@ -562,7 +569,7 @@ int *width2byte2(const char *str, int n)
   static int rval[2];
 
   for (; *str != '\0'; str++) {
-    if (isascii(*str)) {
+    if (isascii((unsigned char)*str)) {
       char_width = 1;
       char_byte = 1;
     } else {
@@ -614,10 +621,11 @@ int strhead(char *str, int n)
  * needleʸξNULL֤
  * needleNULLΤȤNULL֤
  */
-char *rstrstr(char *haystack, const char *needle)
+char *rstrstr_len(const char *haystack, const char *needle, int haystack_len)
 {
-  char *str = NULL;
+  const char *str = NULL;
   int needle_len;
+  const char *new_haystack = haystack;
   assert(haystack != NULL);
   if (needle == NULL) {
     return NULL;
@@ -626,14 +634,78 @@ char *rstrstr(char *haystack, const char *needle)
   if (needle_len <= 0) {
     return NULL;
   }
-  while ((haystack = strstr(haystack, needle)) != NULL) {
-    haystack += needle_len;
-    str = haystack;
+  while ((new_haystack = strstr_len(haystack, needle, haystack_len)) != NULL) {
+    new_haystack += needle_len;
+    haystack_len -= (new_haystack - haystack);
+    str = haystack = new_haystack;
   }
-  return str;
+  return (char *)str;
+}
+
+/*
+ * haystack'\0'ޤޤƤ褤strstr
+ * haystackĹhaystack_len
+ * haystackneedleNULLǤʤ
+ * needle""ΤȤhaystack֤
+ */
+char *strstr_len(const char *haystack, const char *needle, int haystack_len)
+{
+  int needle_len;
+  int i, j;
+
+  assert(haystack != NULL && needle != NULL);
+
+  needle_len = strlen(needle);
+
+  for (i = 0; i < haystack_len - needle_len + 1; i++) {
+    for (j = 0; j < needle_len; j++) {
+      if (haystack[i + j] != needle[j]) {
+        break;
+      }
+    }
+    if (j == needle_len) {
+      return (char *)haystack + i;
+    }
+  }
+  return NULL;
 }
 
 static int min(int a, int b)
 {
   return a < b ? a : b;
 }
+
+#define TAB_WIDTH 4
+/*
+ * tabstrΥ֤TAB_WIDTHĤΥڡ֤롣
+ * ֤ͤfree롣
+ */
+char *tab2space(const char *tabstr)
+{
+  char *spacestr;
+  int tabstr_len = strlen(tabstr);
+  int i, j;
+  int tabcount = 0;
+
+  for (i = 0; i < tabstr_len; i++) {
+    if (tabstr[i] == '\t') {
+      tabcount++;
+    }
+  }
+
+  spacestr = malloc((tabstr_len - tabcount) + (TAB_WIDTH * tabcount) + 1);
+
+  for (i = 0, j = 0; i < tabstr_len + 1; i++, j++) {
+    if (tabstr[i] == '\t') {
+      int i2;
+      for (i2 = 0; i2 < TAB_WIDTH; i2++, j++) {
+        spacestr[j] = ' ';
+      }
+      j--;
+    } else {
+      spacestr[j] = tabstr[i];
+    }
+  }
+
+  return spacestr;
+}
