@@ -159,8 +159,17 @@ static void
 im_uim_commit_string(void *ptr, const char *str)
 {
   IMUIMContext *uic = (IMUIMContext *)ptr;
+  uim_bool show_state;
+  gint x, y;
+
   g_return_if_fail(str);
   g_signal_emit_by_name(uic, "commit", str);
+
+  show_state = uim_scm_symbol_value_bool("bridge-show-input-state?");
+  if (show_state == UIM_TRUE) {
+    gdk_window_get_origin(uic->win, &x, &y);
+    caret_state_indicator_update(uic->caret_state_indicator, x, y, NULL);
+  }
 }
 
 static void
@@ -506,6 +515,7 @@ focus_out(GtkIMContext *ic)
   if (uic->cwin) {
     gtk_widget_hide(GTK_WIDGET(uic->cwin));
   }
+  gtk_widget_hide(uic->caret_state_indicator);
 }
 
 static void
@@ -655,6 +665,12 @@ im_uim_finalize(GObject *obj)
     gtk_widget_destroy(GTK_WIDGET(uic->cwin));
     uic->cwin = NULL;
   }
+  if (uic->caret_state_indicator) {
+    guint tag = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(uic->caret_state_indicator), "timeout-tag"));
+    g_source_remove(tag);
+    gtk_widget_destroy(uic->caret_state_indicator);
+    uic->caret_state_indicator = NULL;
+  }
 
   uim_release_context(uic->uc);
 
@@ -700,7 +716,7 @@ update_prop_list_cb(void *ptr, const char *str)
   IMUIMContext *uic = (IMUIMContext *)ptr;
   GString *tmp;
 
-  if (uic != focused_context)
+  if (uic != focused_context || disable_focused_context)
     return;
 
   tmp = g_string_new("");
@@ -718,7 +734,7 @@ update_prop_label_cb(void *ptr, const char *str)
   gint x, y;
   uim_bool show_state;
 
-  if (uic != focused_context)
+  if (uic != focused_context || disable_focused_context)
     return;
 
   tmp = g_string_new("");
@@ -729,8 +745,13 @@ update_prop_label_cb(void *ptr, const char *str)
 
   show_state = uim_scm_symbol_value_bool("bridge-show-input-state?");
   if (show_state == UIM_TRUE) {
+    gint timeout;
     gdk_window_get_origin(uic->win, &x, &y);
     caret_state_indicator_update(uic->caret_state_indicator, x, y, str);
+    timeout = uim_scm_symbol_value_int("bridge-show-input-state-time-length");
+    if(timeout != 0)
+      caret_state_indicator_set_timeout(uic->caret_state_indicator, timeout * 1000);
+    gtk_widget_show_all(uic->caret_state_indicator);
   }
 }
 
@@ -933,27 +954,34 @@ im_uim_parse_helper_str_im_change(const char *str)
   IMUIMContext *cc;
   gchar **lines = g_strsplit(str, "\n", -1);
   gchar *im_name = lines[1];
+  GString *im_name_sym = g_string_new(im_name);
+  g_string_prepend_c(im_name_sym, '\'');
   if (g_str_has_prefix(str, "im_change_this_text_area_only") == TRUE) {
-    if (focused_context) {
+    if (focused_context && disable_focused_context == FALSE) {
       uim_switch_im(focused_context->uc, im_name);
       uim_prop_list_update(focused_context->uc);
     }
   } else if (g_str_has_prefix(str, "im_change_whole_desktop") == TRUE) {
     for (cc = context_list.next; cc != &context_list; cc = cc->next) {
       uim_switch_im(cc->uc, im_name);
+      uim_prop_update_custom(cc->uc, "custom-preserved-default-im-name",
+			     im_name_sym->str);
       if (focused_context && cc == focused_context)
 	uim_prop_list_update(cc->uc);
     }
   } else if (g_str_has_prefix(str, "im_change_this_application_only") == TRUE) {
-    if (focused_context) {    
+    if (focused_context && disable_focused_context == FALSE) {
       for (cc = context_list.next; cc != &context_list; cc = cc->next) {
 	uim_switch_im(cc->uc, im_name);
+	uim_prop_update_custom(cc->uc, "custom-preserved-default-im-name",
+			       im_name_sym->str);
 	if (cc == focused_context)
 	  uim_prop_list_update(cc->uc);
       }
     }
   }
   g_strfreev(lines);
+  g_string_free(im_name_sym, TRUE);
 }
 
 static void
