@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2004-2005 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -17,10 +17,10 @@
      may be used to endorse or promote products derived from this software
      without specific prior written permission.
 
-  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
@@ -47,7 +47,7 @@
 
 #define BUTTON_H_ALIGN 5
 
-gchar* alphabet_capital[] = {
+gchar *alphabet_capital[] = {
   "A", "B", "C", "D", "E",
   "F", "G", "H", "I", "J",
   "K", "L", "M", "N", "O",
@@ -56,7 +56,7 @@ gchar* alphabet_capital[] = {
   "Z", NULL
 };
 
-gchar* alphabet_small[] = {
+gchar *alphabet_small[] = {
   "a", "b", "c", "d", "e",
   "f", "g", "h", "i", "j",
   "k", "l", "m", "n", "o",
@@ -65,13 +65,13 @@ gchar* alphabet_small[] = {
   "z", NULL
 };
 
-gchar* numbers[] = {
+gchar *numbers[] = {
   "0", "1", "2", "3", "4",
   "5", "6", "7", "8", "9",
   NULL
 };
 
-gchar* symbols[] = {
+gchar *symbols[] = {
   "@",  ":", "_", "~", "/",
   "\\", "(", ")", "'", "\"",
   "-",  ",", ".", "!", "?",
@@ -127,18 +127,103 @@ gchar *hiragana[] = {
   NULL
 };
 
+gchar *kigou[] = {
+  "○", "●", "◎", "♂", "♀",
+  "☆", "★", "□", "■", "△",
+  "▲", "▽", "▼", "〒",
+  NULL
+};
+
+gchar *bracket[] = {
+  "‘", "’", "“", "”", "（",
+  "）", "〔", "〕", "［", "］",
+  "｛", "｝", "〈", "〉", "《",
+  "》", "「", "」", "『", "』",
+  "【", "】", "〝", "〟",
+  NULL
+};
+
+gchar *arrow[] = {
+  "→", "←", "↑", "↓",
+  NULL
+};
+
+gchar *omission[] = {
+  "㍻", "№", "㏍", "℡", "㊤",
+  "㊥", "㊦", "㊧", "㊨", "㈱",
+  "㈲", "㈹", "㍾", "㍽", "㍼",
+  "™", "©", "®",
+  NULL
+};
+
+gchar *unit[] = {
+  "℃", "￥", "＄", "£",
+  "％", "㍉", "㌔", "㌢", "㍍",
+  "㌘", "㌧", "㌃", "㌶", "㍑",
+  "㍗", "㌍", "㌦", "㌣", "㌫",
+  "㍊", "㌻", "㎜", "㎝", "㎞",
+  "㎎", "㎏", "㏄", "ℓ",  "㎟",
+  "㎠", "㎡", "㎢", "㎣", "㎤",
+  "㎥", "㎦", "Å", "‰",
+};
+
+gchar *dot[] = {
+  "、", "。", "，", "．", "・",
+  "：", "；", "？", "！", "゛",
+  "゜", "´", "｀", "¨", "＾",
+  "ヽ", "ヾ", "ゝ", "ゞ", "〃",
+  "…", "‥", "°", "′", "″",
+  NULL
+};
+
+gchar *academic[] = {
+  "＋", "－", "±", "×", "÷",
+  "＝", "≠", "＜", "＞", "≦",
+  "≧", "∞", "∴", "≒", "≡",
+  "∫", "∮", "∑", "√", "⊥",
+  "∠", "∟", "⊿", "∵", "∩",
+  "∪", "∈", "∋", "⊆", "⊇",
+  "⊂", "⊃", "∧", "∨", "￢",
+  "⇒", "⇔", "∀", "∃", "⌒",
+  "∂", "∇", "≪", "≫", "∽",
+  "∝", "∬",
+  NULL
+};
+
+gchar *number[] = {
+  "①", "②", "③", "④", "⑤",
+  "⑥", "⑦", "⑧", "⑨", "⑩",
+  "⑪", "⑫", "⑬", "⑭", "⑮",
+  "⑯", "⑰", "⑱", "⑲", "⑳",
+  "Ⅰ", "Ⅱ", "Ⅲ", "Ⅳ", "Ⅴ",
+  "Ⅶ", "Ⅷ", "Ⅸ", "Ⅹ", "ⅰ",
+  "ⅱ", "ⅲ", "ⅳ", "ⅴ", "ⅵ",
+  "ⅶ", "ⅷ", "ⅸ", "ⅹ",
+};
+
+gchar *line[] = {
+  "─", "│", "┌", "┐", "┘",
+  "└", "├", "┬", "┤", "┴",
+  "┴", "┼", "━", "┏", "┓",
+  "┛", "┗", "┣", "┳", "┫",
+  "┻", "╋", "┠", "┯", "┨",
+  "┷", "┿", "┝", "┰", "┥",
+  "┸", "╂",
+  NULL
+};
+
 static int uim_fd = -1;
 
-static GtkWidget* buttontable_create(char **table, int tablelen);
-static GtkWidget* create_tab1(void);
-static GtkWidget* create_tab2(void);
-static GtkWidget* create_tab3(void);
+static GtkWidget *buttontable_create(char **table, int tablelen);
+static GtkWidget *create_hiragana_tab(void);
+static GtkWidget *create_katakana_tab(void);
+static GtkWidget *create_eisu_tab(void);
 
-static void         check_helper_connection(void);
-static void         helper_disconnect_cb(void);
-static void         input_pad_create(void);
-static GtkWidget *  input_table_create(gchar *localename);
-static void         padbutton_clicked(GtkButton *button, gpointer user_data);
+static void       check_helper_connection(void);
+static void       helper_disconnect_cb(void);
+static void       input_pad_create(void);
+static GtkWidget *input_table_create(gchar *localename);
+static void       padbutton_clicked(GtkButton *button, gpointer user_data);
 
 
 static void
@@ -167,15 +252,15 @@ padbutton_clicked(GtkButton *button, gpointer user_data)
 
   tmp = g_string_new("commit_string\n");
   g_string_append(tmp, str);
-  g_string_append(tmp, "\n\n");
+  g_string_append(tmp, "\n");
 
   uim_helper_send_message(uim_fd, tmp->str);
 
   g_string_free(tmp, TRUE);
 }
 
-static GtkWidget*
-buttontable_create(gchar** table, int len)
+static GtkWidget *
+buttontable_create(gchar **table, int len)
 {
   GtkWidget *_table;
   GtkWidget *button;
@@ -188,11 +273,11 @@ buttontable_create(gchar** table, int len)
   gtk_table_set_row_spacings(GTK_TABLE(_table), 3);
   gtk_table_set_col_spacings(GTK_TABLE(_table), 3);
 
-  for(i=0; i < rows; i++) {
-    for(j=0; j < BUTTON_H_ALIGN; j++) {
-      if(table[i*BUTTON_H_ALIGN + j] == NULL)
+  for (i=0; i < rows; i++) {
+    for (j=0; j < BUTTON_H_ALIGN; j++) {
+      if (table[i*BUTTON_H_ALIGN + j] == NULL)
 	goto out;
-      if(strcmp(table[i*BUTTON_H_ALIGN + j], "") == 0)
+      if (strcmp(table[i*BUTTON_H_ALIGN + j], "") == 0)
 	continue;
 
       button = gtk_button_new_with_label(table[i*BUTTON_H_ALIGN + j]);
@@ -209,8 +294,22 @@ buttontable_create(gchar** table, int len)
   return _table;
 }
 
-static GtkWidget*
-create_tab1()
+static GtkWidget *
+create_tab(gchar *table[], guint len)
+{
+  GtkWidget *vbox;
+
+  vbox = gtk_vbox_new(FALSE, 10);
+
+  gtk_box_pack_start(GTK_BOX(vbox),
+		     buttontable_create(table, len),
+		     FALSE, FALSE, 0);
+
+  return vbox;
+}
+
+static GtkWidget *
+create_hiragana_tab(void)
 {
   GtkWidget *vbox;
 
@@ -223,8 +322,8 @@ create_tab1()
   return vbox;
 }
 
-static GtkWidget*
-create_tab2()
+static GtkWidget *
+create_katakana_tab(void)
 {
   GtkWidget *vbox;
 
@@ -237,8 +336,8 @@ create_tab2()
   return vbox;
 }
 
-static GtkWidget*
-create_tab3()
+static GtkWidget *
+create_eisu_tab(void)
 {
   GtkWidget *vbox;
 
@@ -282,21 +381,49 @@ input_pad_create(void)
 
 }
 
-static GtkWidget*
+static GtkWidget *
 input_table_create(gchar *localename)
 {
   GtkWidget *notebook;
   
   notebook = gtk_notebook_new();
+  gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), TRUE);
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
-			   create_tab1(),
-			   gtk_label_new(_("hira")));
+			   create_hiragana_tab(),
+			   gtk_label_new(_("hiragana")));
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
-			   create_tab2(),
-			   gtk_label_new(_("kata")));
+			   create_katakana_tab(),
+			   gtk_label_new(_("katakana")));
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
-			   create_tab3(),
+			   create_eisu_tab(),
 			   gtk_label_new(_("eisu")));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
+			   create_tab(kigou, sizeof(kigou)/sizeof(gchar*)),
+			   gtk_label_new(_("symbol")));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
+			   create_tab(bracket, sizeof(bracket)/sizeof(gchar*)),
+			   gtk_label_new(_("bracket")));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
+			   create_tab(arrow, sizeof(arrow)/sizeof(gchar*)),
+			   gtk_label_new(_("arrow")));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
+			   create_tab(omission, sizeof(omission)/sizeof(gchar*)),
+			   gtk_label_new(_("omission")));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
+			   create_tab(unit, sizeof(unit)/sizeof(gchar*)),
+			   gtk_label_new(_("unit")));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
+			   create_tab(dot, sizeof(dot)/sizeof(gchar*)),
+			   gtk_label_new(_("dot")));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
+			   create_tab(number, sizeof(number)/sizeof(gchar*)),
+			   gtk_label_new(_("number")));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
+			   create_tab(academic, sizeof(academic)/sizeof(gchar*)),
+			   gtk_label_new(_("academic")));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
+			   create_tab(line, sizeof(line)/sizeof(gchar*)),
+			   gtk_label_new(_("line")));
   return notebook;
 }
 
