@@ -221,7 +221,7 @@ void QUimHelperManager::helper_disconnect_cb()
 void QUimHelperManager::update_prop_list_cb( void *ptr, const char *str )
 {
     QUimInputContext *ic = ( QUimInputContext* ) ptr;
-    if ( ic != focusedInputContext )
+    if ( ic != focusedInputContext || disableFocusedContext )
         return;
 
     QString msg = "prop_list_update\ncharset=UTF-8\n";
@@ -233,7 +233,7 @@ void QUimHelperManager::update_prop_list_cb( void *ptr, const char *str )
 void QUimHelperManager::update_prop_label_cb( void *ptr, const char *str )
 {
     QUimInputContext *ic = ( QUimInputContext* ) ptr;
-    if ( ic != focusedInputContext )
+    if ( ic != focusedInputContext || disableFocusedContext )
         return;
 
     QString msg = "prop_label_update\ncharset=UTF-8\n";
