@@ -291,7 +291,7 @@
 	   (extract-kana
 	    (if (= rule anthy-input-rule-kana)
 		(lambda (entry) (car entry))
-		(lambda (entry) (nth kana entry)))))
+		(lambda (entry) (list-ref entry kana)))))
 
       (string-append
        (string-append-map-ustr-former extract-kana preconv-str)
@@ -346,7 +346,8 @@
 (define anthy-release-handler
   (lambda (ac)
     (let ((ac-id (anthy-context-ac-id ac)))
-      (anthy-lib-free-context ac-id))))
+      (if (number? ac-id)
+	  (anthy-lib-free-context ac-id)))))
 
 (define anthy-flush
   (lambda (ac)
@@ -609,7 +610,7 @@
        ;; modifiers (except shift) => ignore
        ((and (modifier-key-mask key-state)
 	     (not (shift-key-mask key-state)))
-	(anthy-commit-raw ac))
+	  #f)
 
        (else	
 	(let* ((key-str (charcode->string 
@@ -712,7 +713,7 @@
 	   (extract-kana
 	    (if (= rule anthy-input-rule-kana)
 		(lambda (entry) (car entry))
-		(lambda (entry) (nth kana entry)))))
+		(lambda (entry) (list-ref entry kana)))))
 
       (list
        (and (not (ustr-cursor-at-beginning? preconv-str))
