@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -15,10 +15,10 @@
 ;;;    may be used to endorse or promote products derived from this software
 ;;;    without specific prior written permission.
 ;;;
-;;; THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+;;; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND
 ;;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE
 ;;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 ;;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 ;;; OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
@@ -32,9 +32,9 @@
 ;;
 
 (define-record 'skk-editor
-  '((context      ())
-    (left-string  ())
-    (right-string ())))
+  '((context      '())
+    (left-string  '())
+    (right-string '())))
 (define skk-editor-new-internal skk-editor-new)
 
 (define skk-editor-new
@@ -127,22 +127,48 @@
 	       (begin
 		 (if (< 0 (string-length str))
 		     (begin
-		       (skk-lib-learn-word
-			(skk-make-string (skk-context-head sc)
-					 skk-type-hiragana)
-			(skk-context-okuri-head sc)
-			(skk-make-string (skk-context-okuri sc)
-					 skk-type-hiragana)
-			str)
-		       (skk-save-personal-dictionary)))
-		 (skk-commit-editor-context sc str)
+		       (if skk-use-numeric-conversion?
+			   (skk-lib-learn-word
+			    (skk-lib-replace-numeric (skk-make-string
+			    			      (skk-context-head sc)
+						      skk-type-hiragana))
+			    (skk-context-okuri-head sc)
+			    (skk-make-string (skk-context-okuri sc)
+					     skk-type-hiragana)
+			    str)
+			   (skk-lib-learn-word
+			    (skk-make-string (skk-context-head sc)
+					     skk-type-hiragana)
+			    (skk-context-okuri-head sc)
+			    (skk-make-string (skk-context-okuri sc)
+					     skk-type-hiragana)
+			    str))
+		       (skk-save-personal-dictionary)
+		       (if skk-use-numeric-conversion?
+			  (let ((numlst
+				 (skk-lib-store-replaced-numstr
+				  (skk-make-string (skk-context-head sc)
+						   skk-type-hiragana))))
+			    (set! str (skk-lib-merge-replaced-numstr
+				       str numlst))))
+		       (skk-commit-editor-context sc str))
+		     (begin
+		       (skk-editor-flush ec)
+		       (skk-context-set-child-context! sc '())
+		       (skk-context-set-child-type! sc '())
+		       (if (> (skk-context-nr-candidates sc) 0)
+			   (skk-back-to-converting-state sc)
+			   (skk-back-to-kanji-state sc))))
 		 #f)
 	       #t)
 	   (if (skk-cancel-key? key key-state)
 	       (begin
 		 (skk-editor-flush ec)
-		 (skk-back-to-kanji-state sc)
-		 (skk-context-set-child-context! sc #f)
+		 (skk-context-set-child-context! sc '())
+		 (skk-context-set-child-type! sc '())
+		 (if (> (skk-context-nr-candidates sc) 0)
+		     (skk-back-to-converting-state sc)
+		     (skk-back-to-kanji-state sc))
 		 #f)
 	       #t)
 	   )))))
