@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -17,10 +17,10 @@
      may be used to endorse or promote products derived from this software
      without specific prior written permission.
 
-  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
@@ -33,12 +33,15 @@
 #ifndef _context_h_included_
 #define _context_h_included_
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #include <stdio.h>
 #include "config.h"
 #include "gettext.h"
 #include "uim.h"
 #include "uim-scm.h"
-#include "siod.h"
 
 struct uim_im {
   char *name;
@@ -47,12 +50,6 @@ struct uim_im {
   char *short_desc;
 };
 
-struct cb_q {
-  struct cb *first_cb;
-  struct cb *tail_cb;
-  int flushing;
-};
-
 struct uim_candidate_ {
   char *str;         /* candidate */
   char *heading_label;
@@ -117,47 +114,24 @@ struct uim_context_ {
   /* surrounding text */
   void (*request_surrounding_text_cb)(void *ptr);
   int (*delete_surrounding_text_cb)(void *ptr, int offset, int len);
-  /* callback queue */
-  struct cb_q cb_q;
   /* preedit segments array */
   struct preedit_segment *psegs;
   int nr_psegs;
 };
 
-enum {
-  CAND_ACTIVATE_CB,
-  CAND_SELECT_CB,
-  CAND_DEACTIVATE_CB,
-  COMMIT_CB,
-  PREEDIT_CLEAR_CB,
-  PREEDIT_PUSHBACK_CB,
-  PREEDIT_UPDATE_CB,
-  MODE_UPDATE_CB,
-  MODE_LIST_UPDATE_CB,
-  PROP_LABEL_UPDATE_CB,
-  PROP_LIST_UPDATE_CB,
-  CAND_SHIFT_PAGE_CB,
-  REQUEST_SURROUNDING_CB,
-  DELETE_SURROUNDING_CB
-};
-
-struct cb {
-  int type;
-  char *str;
-  int n1, n2;
-  struct cb *next;
-};
-
 
 #ifdef ENABLE_NLS
 #define UIM_PREPARE_SAVING_TEXTDOMAIN_CODESET() \
-    const char *orig_encoding, *client_encoding;
+    char *enc, *orig_encoding = NULL; \
+    const char *client_encoding;
 #define UIM_SWITCH_TEXTDOMAIN_CODESET(uc) \
-  orig_encoding = bind_textdomain_codeset(GETTEXT_PACKAGE, NULL); \
+  if ((enc = bind_textdomain_codeset(GETTEXT_PACKAGE, NULL))) \
+    orig_encoding = strdup(enc); \
   client_encoding = (uc) ? ((struct uim_context_ *)uc)->encoding : uim_last_client_encoding; \
   bind_textdomain_codeset(GETTEXT_PACKAGE, client_encoding);
 #define UIM_RESTORE_TEXTDOMAIN_CODESET() \
-  bind_textdomain_codeset(GETTEXT_PACKAGE, orig_encoding);
+  bind_textdomain_codeset(GETTEXT_PACKAGE, orig_encoding); \
+  free(orig_encoding);
 #else  /* ENABLE_NLS */
 #define UIM_PREPARE_SAVING_TEXTDOMAIN_CODESET()
 #define UIM_SWITCH_TEXTDOMAIN_CODESET(uc)
@@ -171,7 +145,7 @@ struct cb {
       if (uc) \
         uim_eval_string(uc, sexp_str); \
       else \
-        repl_c_string(sexp_str, 0, 1); \
+        uim_scm_eval_c_string(sexp_str); \
 
 #define UIM_EVAL_STRING(uc, sexp_str) \
   { \
@@ -264,24 +238,23 @@ struct cb {
 /**/
 uim_context
 uim_find_context(int id);
+void uim_scm_init(const char *verbose_level);
+void uim_scm_quit(void);
+
+#ifdef UIM_COMPAT_SCM
+void uim_init_compat_scm_subrs(void);
+#endif
 void uim_init_key_subrs(void);
 void uim_init_util_subrs(void);
+#ifdef UIM_COMPAT_TABLE
 void uim_init_table_subrs(void);
+#endif
 void uim_init_im_subrs(void);
+void uim_init_intl_subrs(void);
 
 /**/
-void uim_init_skk_dic(void);
-void uim_quit_skk_dic(void);
-void uim_init_anthy(void);
-void uim_quit_anthy(void);
-void uim_init_prime(void);
-void uim_quit_prime(void);
-void uim_init_m17nlib(void);
-void uim_quit_m17nlib(void);
-
-/**/
-char *uim_get_c_string(LISP str);
-int uim_key_sym_to_int(LISP sym);
+void uim_init_plugin(void);
+void uim_quit_plugin(void);
 
 int uim_iconv_is_convertible(const char *tocode, const char *fromcode);
 void *uim_iconv_create(const char *tocode, const char *fromcode);
@@ -290,7 +263,6 @@ void uim_iconv_release(void *obj);
 
 int uim_sizeof_sexp_str(const char *tmpl, ...);
 void uim_eval_string(uim_context, char *str);
-void uim_schedule_cb(uim_context, int type, char *str, int n1, int n2);
 void uim_release_preedit_segments(uim_context uc);
 void uim_update_preedit_segments(uim_context uc);
 
@@ -298,4 +270,7 @@ extern struct uim_im *uim_im_array;
 extern int uim_nr_im;
 extern char *uim_last_client_encoding;
 
+#ifdef __cplusplus
+}
+#endif
 #endif
