@@ -1,6 +1,8 @@
 /*
 
-  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+  uim-ipc.c: Utility functions for inter process communication.
+
+  Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
 
   All rights reserved.
 
@@ -17,10 +19,10 @@
      may be used to endorse or promote products derived from this software
      without specific prior written permission.
 
-  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
@@ -44,6 +46,11 @@
 #include "context.h"
 #include "uim-helper.h"
 
+#ifndef HAVE_STRSEP
+char *uim_strsep(char **stringp, const char *delim);
+#define strsep	uim_strsep
+#endif
+
 /* This function is come from the GNU C Library manual */
 static int
 set_cloexec(int fd)
@@ -140,10 +147,12 @@ open_pipe_rw(FILE **fr, FILE **fw)
 }
 
 int
-uim_ipc_open_command(int old_pid, FILE **read_fp,
-		     FILE **write_fp, char *command)
+uim_ipc_open_command_with_option(int old_pid, FILE **read_fp,
+		     FILE **write_fp, const char *command, const char *option)
 {
   int new_pid, result;
+  char **ap, *argv[10];
+  char *str = NULL, *p;
 
   if (*read_fp != NULL) {
     fclose(*read_fp);
@@ -175,34 +184,59 @@ uim_ipc_open_command(int old_pid, FILE **read_fp,
       set_cloexec(i);      
     }
 
-    if(is_setugid() != 0) {
+    if (!option) {
+      argv[0] = (char *)command;
+      argv[1] = NULL;
+    } else {
+      argv[0] = (char *)command;
+      str = p = strdup(option);
+      for (ap = &argv[1]; (*ap = strsep(&p, " ")) != NULL;) {
+	if (**ap != '\0')
+	  if (++ap >= &argv[9])
+	    break;
+      }
+      *ap = NULL;
+    }
+    if (is_setugid() != 0) {
       int cmd_len = strlen(command) + 30;
       char *fullpath_command = malloc(cmd_len);
-
-      /*if(setuid(getuid())!=0) abort();*/ /* discarding privilege */
-
+      
+      /*if (setuid(getuid())!=0) abort();*/ /* discarding privilege */
+      
       snprintf(fullpath_command, cmd_len, "/usr/local/bin/%s", command);
-      result = execlp(fullpath_command, fullpath_command, NULL);
-      if(result == -1) {
-	snprintf(fullpath_command, cmd_len, "/usr/bin/%s", command);
-	result = execlp(fullpath_command, fullpath_command, NULL);
+
+      result = execvp(fullpath_command, argv);
+
+      if (result == -1) {
+ 	snprintf(fullpath_command, cmd_len, "/usr/bin/%s", command);
+	result = execvp(fullpath_command, argv);
       }
       free(fullpath_command);
     } else {
-      result = execlp(command, command, NULL);
+      result = execvp(command, argv);
     }
-    if(result == -1) {
+    free(str);
+
+    if (result == -1) {
       write(1,"err",strlen("err"));
     }
     _exit(127);
   }
+
   return new_pid;
 }
 
+int
+uim_ipc_open_command(int old_pid, FILE **read_fp,
+		     FILE **write_fp, const char *command)
+{
+  return uim_ipc_open_command_with_option(old_pid, read_fp, write_fp, command, NULL);
+}
+
 char *
 uim_ipc_send_command(int *pid,
 		     FILE **read_fp, FILE **write_fp,
-		     char *command, char *str)
+		     const char *command, const char *str)
 {
   char *tmp = strdup("");
   char buf[8192];
@@ -210,7 +244,7 @@ uim_ipc_send_command(int *pid,
   if (*read_fp == NULL || *write_fp == NULL) {
     *pid = uim_ipc_open_command(*pid, read_fp, write_fp, command);
   }
-  if(*pid == 0) {
+  if (*pid == 0) {
     free(tmp);
     return NULL;
   }
@@ -237,7 +271,7 @@ uim_ipc_send_command(int *pid,
   }
 
   while (fgets (buf, sizeof(buf), *read_fp) != NULL) {
-    if(strcmp( buf, "\n" ) == 0) {
+    if (strcmp( buf, "\n" ) == 0) {
       break;
     }
 
