@@ -436,6 +436,111 @@ eucjp_string_to_list(uim_lisp str_)
   return res;
 }
 
+static uim_lisp
+string_prefixp_internal(uim_lisp prefix_, uim_lisp str_,
+			int cmp(const char *, const char *, size_t))
+{
+  const char *prefix, *str;
+  size_t len;
+
+  if (!uim_scm_stringp(prefix_) || !uim_scm_stringp(str_))
+    return uim_scm_f();
+
+  prefix = uim_scm_refer_c_str(prefix_);
+  str = uim_scm_refer_c_str(str_);
+  len = strlen(prefix);
+
+  return cmp(prefix, str, len) ? uim_scm_f() : uim_scm_t();
+}
+
+static uim_lisp
+string_prefixp(uim_lisp prefix_, uim_lisp str_)
+{
+  return string_prefixp_internal(prefix_, str_, strncmp);
+}
+
+static uim_lisp
+string_prefix_cip(uim_lisp prefix_, uim_lisp str_)
+{
+  return string_prefixp_internal(prefix_, str_, strncasecmp);
+}
+
+static uim_lisp
+shift_elems(uim_lisp lists)
+{
+  uim_lisp elms, rests, list;
+
+  if (uim_scm_nullp(lists))
+    return uim_scm_f();
+
+  elms = rests = uim_scm_null_list();
+  for (; !uim_scm_nullp(lists); lists = uim_scm_cdr(lists)) {
+    list = uim_scm_car(lists);
+    if (uim_scm_nullp(list))
+      return uim_scm_f();
+
+    elms = uim_scm_cons(uim_scm_car(list), elms);
+    rests = uim_scm_cons(uim_scm_cdr(list), rests);
+  }
+
+  return uim_scm_cons(uim_scm_reverse(elms),
+		      uim_scm_reverse(rests));
+}
+
+static uim_lisp
+iterate_lists(uim_lisp mapper, uim_lisp seed, uim_lisp lists)
+{
+  uim_lisp elms, rest, rests, mapped, res, termp, pair, form;
+  uim_bool single_listp;
+
+  single_listp = (uim_scm_length(lists) == 1) ? UIM_TRUE : UIM_FALSE;
+  res = seed;
+  if (single_listp) {
+    rest = uim_scm_car(lists);
+  } else {
+    rests = lists;
+  }
+  do {
+    if (single_listp) {
+      /* fast path */
+      elms = uim_scm_list1(uim_scm_car(rest));
+      rest = uim_scm_cdr(rest);
+    } else {
+      pair = shift_elems(rests);
+      if (FALSEP(pair)) {
+	elms = rests = uim_scm_null_list();
+      } else {
+	elms = uim_scm_car(pair);
+	rests = uim_scm_cdr(pair);
+      }
+    }
+
+    form = uim_scm_list3(mapper,
+			 uim_scm_quote(res),
+			 uim_scm_quote(elms));
+    mapped = uim_scm_eval(form);
+    termp = uim_scm_car(mapped);
+    res = uim_scm_cdr(mapped);
+  } while (FALSEP(termp));
+
+  return res;
+}
+
+static uim_lisp
+find_tail(uim_lisp pred, uim_lisp lst)
+{
+  uim_lisp form, elem;
+
+  for (; !uim_scm_nullp(lst); lst = uim_scm_cdr(lst)) {
+    elem = uim_scm_car(lst);
+    form = uim_scm_list2(pred, uim_scm_quote(elem));
+    if (NFALSEP(uim_scm_eval(form)))
+      return lst;
+  }
+
+  return uim_scm_f();
+}
+
 /* Following is utility functions for C world */
 struct _locale_language_table {
   char *locale;
@@ -473,6 +578,18 @@ uim_get_language_name_from_locale(const char *localename)
   return get_language_name_from_locale(localename);
 }
 
+const char *
+uim_get_language_code_from_language_name(const char *language_name)
+{
+  unsigned int i;
+  for (i = 0; i < NR_LOCALE_LANGUAGE; i++) {
+    if (strcmp(locale_language_table[i].language, language_name) == 0) {
+      return locale_language_table[i].locale;
+    }
+  }
+  return NULL;
+}
+
 static uim_lisp
 lang_code_to_lang_name_raw(uim_lisp code_)
 {
@@ -495,7 +612,7 @@ is_setugidp(void)
 }
 
 void
-uim_init_util_subrs()
+uim_init_util_subrs(void)
 {
   uim_scm_init_subr_0("sys-libdir", sys_libdir);
   uim_scm_init_subr_0("sys-pkglibdir", sys_pkglibdir);
@@ -520,6 +637,10 @@ uim_init_util_subrs()
   uim_scm_init_subr_1("unsetenv", c_unsetenv);
   uim_scm_init_subr_2("string-split", uim_split_string);
   uim_scm_init_subr_1("string-to-list", eucjp_string_to_list);
+  uim_scm_init_subr_2("string-prefix?", string_prefixp);
+  uim_scm_init_subr_2("string-prefix-ci?", string_prefix_cip);
+  uim_scm_init_subr_3("iterate-lists", iterate_lists);
+  uim_scm_init_subr_2("find-tail", find_tail);
   uim_scm_init_subr_1("lang-code->lang-name-raw", lang_code_to_lang_name_raw);
   uim_scm_init_subr_0("is-set-ugid?", is_setugidp);
 }
