@@ -58,6 +58,7 @@ static uim_context context_array[CONTEXT_ARRAY_SIZE];
 struct uim_im *uim_im_array;
 int uim_nr_im;
 static int uim_initialized;
+static int uim_quiting;
 
 void
 uim_set_preedit_cb(uim_context uc,
@@ -236,13 +237,6 @@ uim_switch_im(uim_context uc, const char *engine)
   uc->short_desc = uim_return_str;
   uim_return_str = NULL;  /* ownership has been transferred */
 #endif
-
-  /*
-     To remember the IM selected by uim-toolbar, we record the latest
-     IM to default-im-name. This makes subsequent contexts the
-     selected one. -- YamaKen 2004-11-15
-  */
-  UIM_EVAL_FSTRING1(uc, "(set! default-im-name '%s)", engine);
 }
 
 void
@@ -677,9 +671,11 @@ uim_quit(void)
 {
   int i;
 
-  if (!uim_initialized) {
+  if (!uim_initialized || uim_quiting) {
     return;
   }
+  /* Some multithreaded applications calls uim_quit bursty. */
+  uim_quiting = 1;
 
   /* release still active contexts */
   for (i = 0; i < CONTEXT_ARRAY_SIZE; i++) {
@@ -693,4 +689,5 @@ uim_quit(void)
   free(uim_last_client_encoding);
   uim_last_client_encoding = NULL;
   uim_initialized = 0;
+  uim_quiting = 0;
 }
