@@ -39,8 +39,12 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <errno.h>
+#include <fcntl.h>
 
 #include "uim/uim.h"
+#ifdef UIM_COMPAT_SCM
+#include "uim/uim-compat-scm.h"
+#endif /* UIM_COMPAT_SCM */
 
 #include "ximserver.h"
 #include "xim.h"
@@ -49,10 +53,11 @@
 #include "util.h"
 
 
-static FILE *candwin_r = NULL, *candwin_w = NULL;
-static int candwin_pid = 0;
+static FILE *candwin_r, *candwin_w;
+static int candwin_pid;
 static Canddisp *disp;
 static const char *command;
+static bool candwin_inited = false;
 
 static void candwin_read_cb(int fd, int ev);
 
@@ -60,8 +65,23 @@ static const char *candwin_command(void)
 {
     char *candwin_prog = NULL;
 
+    /*
+      XXX: We should drop uim-compat-scm API. -- omote 07/12/2005
+
+      Search order of candwin_command be summarized as follows
+      if UIM_COMPAT_SCM is defined.
+	 1. UIM_CANDWIN_PROG -- mainly for debugging purpose
+	 2. value in 'uim-candwin-prog' symbol
+	 3. default toolkit's candwin program determined by ./configure
+     */
+
     candwin_prog = getenv("UIM_CANDWIN_PROG");
-    if (candwin_prog == NULL) {
+    if (!candwin_prog) {
+#ifdef UIM_COMPAT_SCM
+	candwin_prog = uim_scm_symbol_value_str("uim-candwin-prog");
+	if (candwin_prog)
+	    return candwin_prog;
+#endif /* UIM_COMPAT_SCM */
 #if defined(USE_QT_CANDWIN)
 	return "uim-candwin-qt";
 #elif defined(USE_GTK_CANDWIN) && defined(USE_GTK2)
@@ -75,15 +95,24 @@ static const char *candwin_command(void)
 
 Canddisp *canddisp_singleton()
 {
-    if (command == NULL)
-	  command = candwin_command();
+    if (!command)
+	command = candwin_command();
 
-    if (disp == NULL && command) {
+    if (!candwin_inited && command) {
 	candwin_pid = uim_ipc_open_command(candwin_pid, &candwin_r, &candwin_w, command);
+	if (disp)
+	    delete disp;
 	disp = new Canddisp();
 	int fd = fileno(candwin_r);
-	if (fd != -1)
-	    add_fd_watch(fd, READ_OK, candwin_read_cb);
+	if (fd != -1) {
+	    int flag = fcntl(fd, F_GETFL);
+	    if (flag != -1) {
+		flag |= O_NONBLOCK;
+		if (fcntl(fd, F_SETFL, flag) != -1)
+		    add_fd_watch(fd, READ_OK, candwin_read_cb);
+	    }
+	}
+	candwin_inited = true;
     }
     return disp;
 }
@@ -99,14 +128,13 @@ void Canddisp::activate(std::vector<const char *> candidates, int display_limit)
 {
     std::vector<const char *>::iterator i;
 
-    if (candwin_w == NULL)
+    if (!candwin_w)
 	return;
 
     fprintf(candwin_w, "activate\ncharset=UTF-8\ndisplay_limit=%d\n",
 		    display_limit);
-    for (i = candidates.begin(); i != candidates.end(); i++) {
+    for (i = candidates.begin(); i != candidates.end(); i++)
 	fprintf(candwin_w, "%s\n", *i);
-    }
     fprintf(candwin_w, "\n");
     fflush(candwin_w);
     check_connection();
@@ -114,7 +142,7 @@ void Canddisp::activate(std::vector<const char *> candidates, int display_limit)
 
 void Canddisp::select(int index)
 {
-    if (candwin_w == NULL)
+    if (!candwin_w)
 	return;
     fprintf(candwin_w, "select\n");
     fprintf(candwin_w, "%d\n\n", index);
@@ -124,7 +152,7 @@ void Canddisp::select(int index)
 
 void Canddisp::deactivate()
 {
-    if (candwin_w == NULL)
+    if (!candwin_w)
 	return;
     fprintf(candwin_w, "deactivate\n\n");
     fflush(candwin_w);
@@ -133,7 +161,7 @@ void Canddisp::deactivate()
 
 void Canddisp::show()
 {
-    if (candwin_w == NULL)
+    if (!candwin_w)
 	return;
     fprintf(candwin_w, "show\n\n");
     fflush(candwin_w);
@@ -142,7 +170,7 @@ void Canddisp::show()
 
 void Canddisp::hide()
 {
-    if (candwin_w == NULL)
+    if (!candwin_w)
 	return;
     fprintf(candwin_w, "hide\n\n");
     fflush(candwin_w);
@@ -151,7 +179,7 @@ void Canddisp::hide()
 
 void Canddisp::move(int x, int y)
 {
-    if (candwin_w == NULL)
+    if (!candwin_w)
 	return;
     fprintf(candwin_w, "move\n");
     fprintf(candwin_w, "%d\n", x);
@@ -161,25 +189,40 @@ void Canddisp::move(int x, int y)
     check_connection();
 }
 
-void Canddisp::show_caret_state(const char *str)
+void Canddisp::show_caret_state(const char *str, int timeout)
 {
-    if (candwin_w == NULL)
+    if (!candwin_w)
 	return;
     fprintf(candwin_w, "show_caret_state\n");
+    fprintf(candwin_w, "%d\n", timeout);
     fprintf(candwin_w, "%s", str);
     fprintf(candwin_w, "\n");
     fflush(candwin_w);
     check_connection();
 }
 
+void Canddisp::update_caret_state()
+{
+    if (!candwin_w)
+	return;
+    fprintf(candwin_w, "update_caret_state\n\n");
+    fflush(candwin_w);
+    check_connection();
+}
+
+void Canddisp::hide_caret_state()
+{
+    if (!candwin_w)
+	return;
+    fprintf(candwin_w, "hide_caret_state\n\n");
+    fflush(candwin_w);
+    check_connection();
+}
+
 void Canddisp::check_connection()
 {
-    if (errno == EBADF || errno == EPIPE) {
-	disp = NULL;
-	int fd = fileno(candwin_r);
-	remove_current_fd_watch(fd);
-	delete this;
-    }
+    if (errno == EBADF || errno == EPIPE)
+	terminate_canddisp_connection();
 }
 
 static void candwin_read_cb(int fd, int ev)
@@ -189,11 +232,7 @@ static void candwin_read_cb(int fd, int ev)
 
     n = read(fd, buf, 1024 - 1);
     if (n == 0) {
-	int fd_w = fileno(candwin_w);
-	if (fd != -1)
-	    close(fd_w);
-	close(fd);
-	remove_current_fd_watch(fd);
+	terminate_canddisp_connection();
 	return;
     }
     if (n == -1)
@@ -201,11 +240,7 @@ static void candwin_read_cb(int fd, int ev)
     buf[n] = '\0';
 
     if (!strcmp(buf, "err")) {
-	int fd_w = fileno(candwin_w);
-	if (fd != -1)
-	    close(fd_w);
-	close(fd);
-	remove_current_fd_watch(fd);
+	terminate_canddisp_connection();
 	return;
     }
 
@@ -247,7 +282,7 @@ void terminate_canddisp_connection()
 	close(fd_w);
     }
 
-    disp = NULL;
-
+    candwin_w = candwin_r = NULL;
+    candwin_inited = false;
     return;
 }
