@@ -104,7 +104,7 @@ bool Compose::handleKey(KeySym xkeysym, int xkeystate, bool is_push)
     } else { // Unmatched
 	if (m_context == m_top)
 	    return false;
-	// Error (Sequence Unmatch occured)
+	// Error (Sequence Unmatch occurred)
 	m_context = m_top;
 	return true;
     }
@@ -654,15 +654,6 @@ XimIM::ParseComposeStringFile(FILE *fp)
     char* tbp;
     struct stat st;
 
-    if (mTreeTop == NULL) {
-	mTreeTop = (DefTree *)malloc(sizeof(DefTree));
-	mTreeTop->succession = NULL;
-	mTreeTop->next = NULL;
-	mTreeTop->mb = NULL;
-	mTreeTop->utf8 = NULL;
-	mTreeTop->ks = NoSymbol;
-    }
-
     if (fstat(fileno(fp), &st) != -1) {
 	unsigned long size = (unsigned long)st.st_size;
 	if (size <= sizeof tb)
