@@ -39,6 +39,7 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <signal.h>
 #include "connection.h"
 #include <list>
 #include <map>
@@ -51,6 +52,10 @@
 #define TRANSPORT_UNIT 20
 #define TRANSPORT_MAX 20	// Emacs's XIM won't work correctly when the value is 100
 
+#ifndef HAVE_SIG_T
+typedef void (*sig_t)(int);
+#endif
+
 extern char *xim_packet_name[];
 
 static std::map<Window, XConnection *> gXConnections;
@@ -101,7 +106,7 @@ reapOldConnection()
 void
 procXClientMessage(XClientMessageEvent *ev)
 {
-    //  check connection request
+    // check connection request
     XimServer *xs = XimServer::findServer(ev->window);
     if (xs && ev->message_type == xim_xconnect) {
 	xs->setupNewConnection(ev);
@@ -398,7 +403,11 @@ void XConnection::writeProc()
     writePassivePacket();
     writeNormalPacket();
 
+    // interrupt while _XFlushInt() here will cause lock up of the display.
+    sig_t old_sigusr1 = signal(SIGUSR1, SIG_IGN);
     XFlush(XimServer::gDpy);
+    signal(SIGUSR1, old_sigusr1);
+
     if (mIsCloseWait) {
 	remove_window_watch(mClientWin);
 	mClientWin = None;
