@@ -61,6 +61,7 @@
 
 Display *XimServer::gDpy;
 std::map<Window, XimServer *> XimServer::gServerMap;
+CandWinPosType XimServer::gCandWinPosType;
 
 // Configuration
 int g_option_mask;
@@ -414,13 +415,18 @@ init_supported_locales()
 {
     std::list<char *> locale_list;
     char *locales;
+    const char *s;
     int len;
 
     asprintf(&supported_locales, "@locale=");
     len = strlen(supported_locales);
 
     // get all locales
-    locales = strdup(compose_localenames_from_im_lang("*"));
+    s = compose_localenames_from_im_lang("*");
+    if (s)
+	locales = strdup(s);
+    else
+	locales = strdup("");
     // replace ':' with ','
     char *sep;
     char *tmp = locales;
@@ -538,6 +544,7 @@ reload_uim(int x)
 	}
     }
 
+    // make sure to use appropriate locale for the focused context
     InputContext *focusedContext = InputContext::focusedContext();
     if (focusedContext)
 	focusedContext->focusIn();
@@ -618,6 +625,7 @@ main(int argc, char **argv)
     if (uim_scm_symbol_value_bool("uim-xim-use-xft-font?"))
 	init_default_xftfont(); // setup Xft fonts for Ov/Rw preedit
 #endif
+    check_candwin_pos_type();
 
     // Handle pending events to prevent hang just after startup
     check_pending_xevent();
