@@ -105,7 +105,6 @@ XimServer::XimServer(const char *name, const char *lang)
 {
     mIMName = strdup(name);
     mIMLang = lang;
-    mUsePreservedDefaultIM = false;
 }
 
 InputContext *XimServer::createContext(XimIC *xic, const char *engine)
@@ -132,39 +131,37 @@ void XimServer::changeContext(const char *engine) {
     for (it = ic_list.begin(); it != ic_list.end(); it++) {
 	(*it)->changeContext(engine);
     }
-    // make sure to update locale of focused context
+    // make sure to use appropriate locale for the focused context
     InputContext *focusedContext = InputContext::focusedContext();
     if (focusedContext)
 	focusedContext->focusIn();
 }
 
 void XimServer::customContext(const char *custom, const char *val) {
-#if 1
-    // Updated ximserver's global im with customized one.
-    // This is temporal hack.
-    //fprintf(stderr, "custom %s, val %s\n", custom, val);
-    if (!strcmp(custom, "custom-activate-default-im-name?")) {
-	if (!strcmp(val, "#t"))
-	    mUsePreservedDefaultIM = true;
-	else
-	    mUsePreservedDefaultIM = false;
-    }
-    if (!strcmp(custom, "custom-preserved-default-im-name") &&
-		    mUsePreservedDefaultIM == true) {
-	val++;
-	set_im(val);
-	val--;
-    }
-#if HAVE_XFT_UTF8_STRING
-    if (!strcmp(custom, "uim-xim-xft-font-name"))
-	update_default_xftfont(val);
-#endif
-#endif
-
     std::list<InputContext *>::iterator it;
     for (it = ic_list.begin(); it != ic_list.end(); it++) {
 	(*it)->customContext(custom, val);
+	break;
     }
+
+    // Updated global IM of XimServer
+    if (!strcmp(custom, "custom-preserved-default-im-name") &&
+		    uim_scm_symbol_value_bool("custom-activate-default-im-name?"))
+	set_im(++val);
+
+#if HAVE_XFT_UTF8_STRING
+    if (!strcmp(custom, "uim-xim-xft-font-name"))
+	update_default_xftfont();
+#endif
+
+    if (!strcmp(custom, "bridge-show-input-state?") &&
+		    !uim_scm_symbol_value_bool("bridge-show-input-state?")) {
+	Canddisp *disp = canddisp_singleton();
+	disp->hide_caret_state();
+    }
+
+    if (!strcmp(custom, "candidate-window-position"))
+	check_candwin_pos_type();
 }
 
 bool
@@ -290,7 +287,7 @@ InputContext::InputContext(XimServer *svr, XimIC *ic, const char *engine)
 {
     mXic = ic;
     m_pe = new pe_stat(this);
-    mConvdisp = 0;
+    mConvdisp = NULL;
     mServer = svr;
     mEngineName = NULL;
     mLocaleName = NULL;
@@ -299,6 +296,7 @@ InputContext::InputContext(XimServer *svr, XimIC *ic, const char *engine)
     mCandwinActive = false;
     mNumPage = 1;
     mDisplayLimit = 0;
+    mCaretStateShown = false;
 }
 
 InputContext::~InputContext()
@@ -307,7 +305,7 @@ InputContext::~InputContext()
 	mFocusedContext = NULL;
 
     if (mConvdisp)
-	mConvdisp->set_pe(0);
+	mConvdisp->set_pe(NULL);
 
     delete m_pe;
     uim_release_context(mUc);
@@ -441,14 +439,28 @@ InputContext::focusIn()
     check_helper_connection();
     uim_helper_client_focus_in(mUc);
     mFocusedContext = this;
+    if (mConvdisp) {
+	mConvdisp->unset_focus();
+	mConvdisp->move_candwin();
+	mConvdisp->update_caret_state();
+    }
     uim_prop_list_update(mUc);	
     uim_prop_label_update(mUc);	
+    if (hasActiveCandwin())
+	candidate_update();
 }
 
 void
 InputContext::focusOut()
 {
     uim_helper_client_focus_out(mUc);
+    if (mFocusedContext == this) {
+	Canddisp *disp = canddisp_singleton();
+	if (isCaretStateShown())
+	    disp->hide_caret_state();
+	if (hasActiveCandwin())
+	    disp->hide();
+    }
 }
 
 XimServer *
@@ -469,7 +481,7 @@ InputContext::commit_cb(void *ptr, const char *str)
     InputContext *ic = (InputContext *)ptr;
     XimIC *xic = ic->get_ic();
 
-    clear_cb(ptr);
+    ic->clear_pe_stat();
     ic->update_preedit();
     xic->commit_string(str);
 }
@@ -477,7 +489,7 @@ InputContext::commit_cb(void *ptr, const char *str)
 void InputContext::clear_cb(void *ptr)
 {
     InputContext *ic = (InputContext *)ptr;
-    ic->clear_preedit();
+    ic->clear_pe_stat();
 }
 
 void InputContext::pushback_cb(void *ptr, int attr, const char *str)
@@ -532,11 +544,18 @@ void InputContext::update_prop_label_cb(void *ptr, const char *str)
       ic->update_prop_label(str);
 }
 
-void InputContext::clear_preedit()
+void InputContext::clear_pe_stat()
 {
     m_pe->clear();
 }
 
+void InputContext::clear_preedit()
+{
+    clear_pe_stat();
+    if (mConvdisp)
+	mConvdisp->clear_preedit();
+}
+
 uString InputContext::get_preedit_string()
 {
     uString str;
@@ -612,11 +631,10 @@ bool InputContext::hasActiveCandwin()
     return mCandwinActive;
 }
 
+// reset
 void InputContext::clear()
 {
     clear_preedit();
-    if (mConvdisp)
-	mConvdisp->clear_preedit();
     candidate_deactivate();
     uim_reset_context(mUc);
 }
@@ -687,6 +705,7 @@ void InputContext::candidate_update()
 
     disp->activate(active_candidates, mDisplayLimit);
     disp->select(current_cand_selection);
+    disp->show();
 }
 
 void InputContext::candidate_select(int index)
@@ -765,8 +784,10 @@ void InputContext::update_prop_label(const char *str)
     free(buf);
     
     if (show_caret_state == UIM_TRUE) {
+	int timeout = uim_scm_symbol_value_int("bridge-show-input-state-time-length");
 	Canddisp *disp = canddisp_singleton();
-	disp->show_caret_state(str);
+	disp->show_caret_state(str, timeout);
+	mCaretStateShown = true;
     }
 }
 
@@ -780,6 +801,11 @@ const char *InputContext::get_locale_name()
     return mLocaleName;
 }
 
+bool InputContext::isCaretStateShown()
+{
+    return mCaretStateShown;
+}
+
 keyState::keyState(XimIC *ic)
 {
     XimIM *im;
@@ -1075,6 +1101,22 @@ void init_modifier_keys() {
     gMod5Mask = check_modifier(Mod5MaskSyms);
 }
 
+
+void
+check_candwin_pos_type()
+{
+    char *candwin_pos_type = uim_scm_symbol_value_str("candidate-window-position");
+
+    if (candwin_pos_type && !strcmp(candwin_pos_type, "left"))
+	XimServer::gCandWinPosType = Left;
+    else if (candwin_pos_type && !strcmp(candwin_pos_type, "right"))
+	XimServer::gCandWinPosType = Right;
+    else
+	XimServer::gCandWinPosType = Caret;
+
+    free(candwin_pos_type);
+}
+
 /*
  * Local variables:
  *  c-indent-level: 4
