@@ -57,6 +57,11 @@ struct pe_ustring {
     uString s;
     int stat;
 };
+typedef enum {
+    Caret,
+    Left,
+    Right
+} CandWinPosType;
 
 // state of preedit.
 // created in the constructor of InputContext, and deleted in the
@@ -82,8 +87,9 @@ void init_convdisp();
 void init_modifier_keys();
 #if HAVE_XFT_UTF8_STRING
 void init_default_xftfont();
-void update_default_xftfont(const char *s);
+void update_default_xftfont();
 #endif
+void check_candwin_pos_type();
 
 
 // for command line option
@@ -168,6 +174,7 @@ public:
     void update_prop_list(const char *str);
     void update_prop_label(const char *str);
     bool hasActiveCandwin();
+    bool isCaretStateShown();
     const char *get_engine_name();
     const char *get_locale_name();
     void changeContext(const char *engine);
@@ -186,15 +193,15 @@ public:
     static void update_prop_label_cb(void *ptr, const char *str);
     static InputContext *focusedContext();
     static void deletefocusedContext();
-protected:
+private:
     void commit_string(char *);
+    void clear_pe_stat();
+
     XimIC *mXic;
     XimServer *mServer;
     pe_stat *m_pe;
     Convdisp *mConvdisp;
     uim_context mUc;
-private:
-    static InputContext *mFocusedContext;
     bool mCandwinActive;
     int mDisplayLimit;
     int mNumPage;
@@ -203,6 +210,9 @@ private:
     std::vector<const char *> active_candidates;
     char *mEngineName;
     char *mLocaleName;
+    bool mCaretStateShown;
+private:
+    static InputContext *mFocusedContext;
 };
 
 class Locale {
@@ -243,12 +253,12 @@ public:
     static XimServer *findServer(Window w);
     static Display *gDpy;
     static std::map<Window, XimServer *> gServerMap;
+    static CandWinPosType gCandWinPosType;
 private:
     Window mSelectionWin;
     Atom mServerAtom;
     char *mIMName;
     const char *mIMLang;
-    bool mUsePreservedDefaultIM;
 };
 
 struct UIMInfo {
